@@ -79,18 +79,17 @@ function configure_kuryr {
         # process.
         iniset "$KURYR_CONFIG" vif_plug_ovs_privileged helper_command privsep-helper
         iniset "$KURYR_CONFIG" vif_plug_linux_bridge_privileged helper_command privsep-helper
+
+        # When running kuryr-daemon or CNI in container we need to set up
+        # some configs.
+        iniset "$KURYR_CONFIG" cni_daemon docker_mode True
+        iniset "$KURYR_CONFIG" cni_daemon netns_proc_dir "/host_proc"
     fi
 
     if is_service_enabled kuryr-daemon; then
         iniset "$KURYR_CONFIG" cni_daemon daemon_enabled True
         iniset "$KURYR_CONFIG" oslo_concurrency lock_path "$KURYR_LOCK_DIR"
         create_kuryr_lock_dir
-        if [ "$KURYR_K8S_CONTAINERIZED_DEPLOYMENT" == "True" ]; then
-            # When running kuryr-daemon in container we need to set up some
-            # configs.
-            iniset "$KURYR_CONFIG" cni_daemon docker_mode True
-            iniset "$KURYR_CONFIG" cni_daemon netns_proc_dir "/host_proc"
-        fi
     fi
 
     create_kuryr_cache_dir
@@ -125,17 +124,12 @@ function generate_containerized_kuryr_resources {
     inicomment "$KURYR_CONFIG" kubernetes ssl_client_crt_file
     inicomment "$KURYR_CONFIG" kubernetes ssl_client_key_file
 
-    # kuryr-controller and kuryr-cni will have tokens in different dirs.
-    KURYR_CNI_CONFIG=${KURYR_CONFIG}-cni
-    cp $KURYR_CONFIG $KURYR_CNI_CONFIG
     iniset "$KURYR_CONFIG" kubernetes token_file /var/run/secrets/kubernetes.io/serviceaccount/token
     iniset "$KURYR_CONFIG" kubernetes ssl_ca_crt_file /var/run/secrets/kubernetes.io/serviceaccount/ca.crt
-    iniset "$KURYR_CNI_CONFIG" kubernetes token_file /etc/kuryr/token
-    iniset "$KURYR_CNI_CONFIG" kubernetes ssl_ca_crt_file /etc/kuryr/ca.crt
 
     # Generate kuryr resources in k8s formats.
     local output_dir="${DATA_DIR}/kuryr-kubernetes"
-    generate_kuryr_configmap $output_dir $KURYR_CONFIG $KURYR_CNI_CONFIG
+    generate_kuryr_configmap $output_dir $KURYR_CONFIG $KURYR_CONFIG
     generate_kuryr_service_account $output_dir
     generate_controller_deployment $output_dir $KURYR_HEALTH_SERVER_PORT
     generate_cni_daemon_set $output_dir $CNI_BIN_DIR $CNI_CONF_DIR
@@ -149,6 +143,11 @@ function run_containerized_kuryr_resources {
     /usr/local/bin/kubectl create -f \
         "${k8s_data_dir}/service_account.yml" \
         || die $LINENO "Failed to create kuryr-kubernetes ServiceAccount."
+
+    if is_service_enabled openshift-master; then
+        # NOTE(dulek): For OpenShift add privileged SCC to serviceaccount.
+        /usr/local/bin/oadm policy add-scc-to-user privileged -n kube-system -z kuryr-controller
+    fi
     /usr/local/bin/kubectl create -f \
         "${k8s_data_dir}/controller_deployment.yml" \
         || die $LINENO "Failed to create kuryr-kubernetes Deployment."
@@ -188,17 +187,12 @@ function copy_tempest_kubeconfig {
     fi
 }
 
-function _lb_state {
-    # Checks Neutron lbaas for the Load balancer state
-    neutron lbaas-loadbalancer-show "$1" | awk '/provisioning_status/ {print $4}'
-}
-
 function create_k8s_api_service {
     # This allows pods that need access to kubernetes API (like the
     # containerized kuryr controller or kube-dns) to talk to the K8s API
     # service
     local service_cidr
-    local router_ip
+    local kubelet_iface_ip
     local lb_name
 
     lb_name='default/kubernetes'
@@ -207,47 +201,25 @@ function create_k8s_api_service {
                              subnet show "$KURYR_NEUTRON_DEFAULT_SERVICE_SUBNET" \
                              -c cidr -f value)
 
+    kubelet_iface_ip=$(openstack port show kubelet-"${HOSTNAME}" -c fixed_ips -f value | cut -d \' -f 2)
+
     k8s_api_clusterip=$(_cidr_range "$service_cidr" | cut -f1)
 
-    neutron lbaas-loadbalancer-create --name "$lb_name" \
-        --vip-address "$k8s_api_clusterip" \
+    create_load_balancer "$lb_name" "$k8s_api_clusterip" \
         "$KURYR_NEUTRON_DEFAULT_SERVICE_SUBNET"
+    create_load_balancer_listener default/kubernetes:443 HTTPS 443 "$lb_name"
+    create_load_balancer_pool default/kubernetes:443 HTTPS ROUND_ROBIN \
+        default/kubernetes:443 "$lb_name"
 
-    # Octavia needs the LB to be active for the listener
-    while [[ "$(_lb_state $lb_name)" != "ACTIVE" ]]; do
-        sleep 1
-    done
-
-    neutron lbaas-listener-create --loadbalancer "$lb_name" \
-        --name default/kubernetes:443 \
-        --protocol HTTPS \
-        --protocol-port 443
-
-    # We must wait for the LB to be active before we can put a Pool for it
-    while [[ "$(_lb_state $lb_name)" != "ACTIVE" ]]; do
-        sleep 1
-    done
-
-    neutron lbaas-pool-create --loadbalancer "$lb_name" \
-        --name default/kubernetes:443 \
-        --listener default/kubernetes:443 \
-        --protocol HTTPS \
-        --lb-algorithm ROUND_ROBIN
-    # We must wait for the pending pool creation update
-    while [[ "$(_lb_state $lb_name)" != "ACTIVE" ]]; do
-        sleep 1
-    done
+    local api_port
     if is_service_enabled openshift-master; then
-        neutron lbaas-member-create  --subnet public-subnet \
-            --address "${HOST_IP}" \
-            --protocol-port 8443 \
-            default/kubernetes:443
+        api_port=8443
     else
-        neutron lbaas-member-create  --subnet public-subnet \
-            --address "${HOST_IP}" \
-            --protocol-port 6443 \
-            default/kubernetes:443
+        api_port=6443
     fi
+
+    create_load_balancer_member "$(hostname)" "$kubelet_iface_ip" "$api_port" \
+        default/kubernetes:443 public-subnet "$lb_name"
 }
 
 function configure_neutron_defaults {
@@ -345,6 +317,14 @@ function configure_neutron_defaults {
     fi
     iniset "$KURYR_CONFIG" neutron_defaults external_svc_net "$ext_svc_net_id"
     iniset "$KURYR_CONFIG" octavia_defaults member_mode "$KURYR_K8S_OCTAVIA_MEMBER_MODE"
+    if [[ "$use_octavia" == "True" ]]; then
+        # Octavia takes a very long time to start the LB in the gate. We need
+        # to tweak the timeout for the LB creation. Let's be generous and give
+        # it up to 20 minutes.
+        # FIXME(dulek): This might be removed when bug 1753653 is fixed and
+        #               Kuryr restarts waiting for LB on timeouts.
+        iniset "$KURYR_CONFIG" neutron_defaults lbaas_activation_timeout 1200
+    fi
 }
 
 function configure_k8s_pod_sg_rules {
@@ -665,6 +645,10 @@ if [[ "$1" == "stack" && "$2" == "extra" ]]; then
     if is_service_enabled openshift-node; then
         prepare_kubelet
         run_openshift_node
+        KURYR_CONFIGURE_BAREMETAL_KUBELET_IFACE=$(trueorfalse True KURYR_CONFIGURE_BAREMETAL_KUBELET_IFACE)
+        if [[ "$KURYR_CONFIGURE_BAREMETAL_KUBELET_IFACE" == "True" ]]; then
+            ovs_bind_for_kubelet "$KURYR_NEUTRON_DEFAULT_PROJECT"
+        fi
     fi
 
     if is_service_enabled kubernetes-api \
@@ -719,33 +703,42 @@ if [[ "$1" == "stack" && "$2" == "extra" ]]; then
                 build_kuryr_containers $CNI_BIN_DIR $CNI_CONF_DIR False
             fi
             generate_containerized_kuryr_resources
-            run_containerized_kuryr_resources
         fi
     fi
 
 elif [[ "$1" == "stack" && "$2" == "test-config" ]]; then
     if is_service_enabled kuryr-kubernetes; then
+        # NOTE(dulek): This is so late, because Devstack's Octavia is unable
+        #              to create loadbalancers until test-config phase.
         create_k8s_router_fake_service
         create_k8s_api_service
+
+        # FIXME(dulek): This is a very late phase to start Kuryr services.
+        #               We're doing it here because we need K8s API LB to be
+        #               created in order to run kuryr-cni container. Thing is
+        #               Octavia is unable to create LB until test-config phase.
+        #               We can revisit this once Octavia's DevStack plugin will
+        #               get improved.
+        if [ "$KURYR_K8S_CONTAINERIZED_DEPLOYMENT" == "True" ]; then
+            run_containerized_kuryr_resources
+        fi
     fi
 fi
 
 if [[ "$1" == "unstack" ]]; then
     KURYR_K8S_CONTAINERIZED_DEPLOYMENT=$(trueorfalse False KURYR_K8S_CONTAINERIZED_DEPLOYMENT)
     if is_service_enabled kuryr-kubernetes; then
-        if [ "$KURYR_K8S_CONTAINERIZED_DEPLOYMENT" == "False" ]; then
-            stop_process kuryr-kubernetes
-        else
+        if [ "$KURYR_K8S_CONTAINERIZED_DEPLOYMENT" == "True" ]; then
             $KURYR_HYPERKUBE_BINARY kubectl delete deploy/kuryr-controller
         fi
+        stop_process kuryr-kubernetes
     elif is_service_enabled kubelet; then
          $KURYR_HYPERKUBE_BINARY kubectl delete nodes ${HOSTNAME}
     fi
-    if [ "$KURYR_K8S_CONTAINERIZED_DEPLOYMENT" == "False" ]; then
-        stop_process kuryr-daemon
-    else
+    if [ "$KURYR_K8S_CONTAINERIZED_DEPLOYMENT" == "True" ]; then
         $KURYR_HYPERKUBE_BINARY kubectl delete ds/kuryr-cni-ds
     fi
+    stop_process kuryr-daemon
 
     if is_service_enabled kubernetes-controller-manager; then
         stop_container kubernetes-controller-manager
