@@ -235,7 +235,14 @@ class LoadBalancerHandler(k8s_base.ResourceEventHandler):
         if not lbaas_state:
             lbaas_state = obj_lbaas.LBaaSState()
 
+        prev_service_pub_ip_info = lbaas_state.service_pub_ip_info
         if self._sync_lbaas_members(endpoints, lbaas_state, lbaas_spec):
+            # For LoadBalancer service type, update k8s-service status with
+            # floating IP address if needed.
+            if (prev_service_pub_ip_info != lbaas_state.service_pub_ip_info
+                    and lbaas_state.service_pub_ip_info is not None):
+                self._update_lb_status(
+                    endpoints, lbaas_state.service_pub_ip_info.ip_addr)
             # REVISIT(ivc): since _sync_lbaas_members is responsible for
             # creating all lbaas components (i.e. load balancer, listeners,
             # pools, members), it is currently possible for it to fail (due
@@ -246,10 +253,19 @@ class LoadBalancerHandler(k8s_base.ResourceEventHandler):
             # required to deal with such situations (e.g. cleanup, or skip
             # failing items, or validate configuration) to prevent annotation
             # being out of sync with the actual Neutron state.
-            self._set_lbaas_state(endpoints, lbaas_state)
+            try:
+                self._set_lbaas_state(endpoints, lbaas_state)
+            except k_exc.K8sResourceNotFound:
+                # Note(yboaron) It's impossible to store neutron resources
+                # in K8S object since object was deleted. In that case
+                # we should rollback all neutron resources.
+                LOG.debug("LoadBalancerHandler failed to store Openstack "
+                          "resources in K8S object (not found)")
+                self.on_deleted(endpoints, lbaas_state)
 
-    def on_deleted(self, endpoints):
-        lbaas_state = self._get_lbaas_state(endpoints)
+    def on_deleted(self, endpoints, lbaas_state=None):
+        if lbaas_state is None:
+            lbaas_state = self._get_lbaas_state(endpoints)
         if not lbaas_state:
             return
         # NOTE(ivc): deleting pool deletes its members
@@ -301,8 +317,18 @@ class LoadBalancerHandler(k8s_base.ResourceEventHandler):
         pool_by_lsnr_port = {(lsnr_by_id[p.listener_id].protocol,
                               lsnr_by_id[p.listener_id].port): p
                              for p in lbaas_state.pools}
-        pool_by_tgt_name = {p.name: pool_by_lsnr_port[p.protocol, p.port]
-                            for p in lbaas_spec.ports}
+        # NOTE(yboaron): Since LBaaSv2 doesn't support UDP load balancing,
+        #              the LBaaS driver will return 'None' in case of UDP port
+        #              listener creation.
+        #              we should consider the case in which
+        #              'pool_by_lsnr_port[p.protocol, p.port]' is missing
+        pool_by_tgt_name = {}
+        for p in lbaas_spec.ports:
+            try:
+                pool_by_tgt_name[p.name] = pool_by_lsnr_port[p.protocol,
+                                                             p.port]
+            except KeyError:
+                continue
         current_targets = {(str(m.ip), m.port) for m in lbaas_state.members}
 
         for subset in endpoints.get('subsets', []):
@@ -410,11 +436,23 @@ class LoadBalancerHandler(k8s_base.ResourceEventHandler):
 
         return changed
 
+    def _is_pool_in_spec(self, pool, lbaas_state, lbaas_spec):
+        # NOTE(yboaron): in order to check if a specific pool is in lbaas_spec
+        # we should:
+        #  1. get the listener that pool is attached to
+        #  2. check if listener's attributes appear in lbaas_spec.
+        for l in lbaas_state.listeners:
+            if l.id != pool.listener_id:
+                continue
+            for port in lbaas_spec.ports:
+                if l.port == port.port and l.protocol == port.protocol:
+                    return True
+        return False
+
     def _remove_unused_pools(self, endpoints, lbaas_state, lbaas_spec):
-        current_pools = {m.pool_id for m in lbaas_state.members}
         removed_ids = set()
         for pool in lbaas_state.pools:
-            if pool.id in current_pools:
+            if self._is_pool_in_spec(pool, lbaas_state, lbaas_spec):
                 continue
             self._drv_lbaas.release_pool(endpoints,
                                          lbaas_state.loadbalancer,
@@ -454,8 +492,9 @@ class LoadBalancerHandler(k8s_base.ResourceEventHandler):
                 loadbalancer=lbaas_state.loadbalancer,
                 protocol=protocol,
                 port=port)
-            lbaas_state.listeners.append(listener)
-            changed = True
+            if listener is not None:
+                lbaas_state.listeners.append(listener)
+                changed = True
         return changed
 
     def _remove_unused_listeners(self, endpoints, lbaas_state, lbaas_spec):
@@ -530,12 +569,8 @@ class LoadBalancerHandler(k8s_base.ResourceEventHandler):
                     if service_pub_ip_info:
                         # if loadbalancerIP should be defined for lbaas,
                         #  associate it to lbaas VIP
-                        # and update k8s-service status with
-                        # loadbalancerIP address
                         self._drv_service_pub_ip.associate_pub_ip(
                             service_pub_ip_info, lb.port_id)
-                        self._update_lb_status(
-                            endpoints, service_pub_ip_info.ip_addr)
                         lbaas_state.service_pub_ip_info = service_pub_ip_info
                 changed = True
             elif lbaas_state.service_pub_ip_info:
