@@ -12,18 +12,21 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+import os
+from six.moves import http_client as httplib
+
 from flask import Flask
+from oslo_config import cfg
+from oslo_log import log as logging
+
 from keystoneauth1 import exceptions as k_exc
 from keystoneclient import client as keystone_client
 from kuryr.lib._i18n import _
 from kuryr.lib import config as kuryr_config
 from kuryr.lib import utils
+from kuryr_kubernetes import clients
+from kuryr_kubernetes import exceptions as exc
 from kuryr_kubernetes.handlers import health as h_health
-import os
-from oslo_config import cfg
-from oslo_log import log as logging
-import requests
-from six.moves import http_client as httplib
 
 LOG = logging.getLogger(__name__)
 CONF = cfg.CONF
@@ -65,11 +68,11 @@ class HealthServer(object):
                 LOG.error(error_message)
                 return error_message, httplib.NOT_FOUND, self.headers
 
-        k8s_conn, status = self.verify_k8s_connection()
+        k8s_conn = self.verify_k8s_connection()
         if not k8s_conn:
             error_message = 'Error when processing k8s healthz request.'
             LOG.error(error_message)
-            return error_message, status, self.headers
+            return error_message, httplib.INTERNAL_SERVER_ERROR, self.headers
         try:
             self.verify_keystone_connection()
         except k_exc.http.HttpError as h_ex:
@@ -109,11 +112,13 @@ class HealthServer(object):
             raise
 
     def verify_k8s_connection(self):
-        path = '/healthz'
-        address = CONF.kubernetes.api_root
-        url = address + path
-        resp = requests.get(url, headers={'Connection': 'close'})
-        return resp.content == 'ok', resp.status_code
+        k8s = clients.get_kubernetes_client()
+        try:
+            k8s.get('/healthz', json=False, headers={'Connection': 'close'})
+        except exc.K8sClientException:
+            LOG.exception('Exception when trying to reach Kubernetes API.')
+            return False
+        return True
 
     def verify_keystone_connection(self):
         conf_group = kuryr_config.neutron_group.name
