@@ -134,18 +134,3 @@ class TestRetryHandler(test_base.TestCase):
         m_sleep.assert_has_calls([
             mock.call(deadline, i + 1, failures[i])
             for i in range(len(failures))])
-
-    @mock.patch('itertools.count')
-    @mock.patch.object(h_retry.Retry, '_sleep')
-    def test_call_should_not_raise(self, m_sleep, m_count):
-        event = mock.sentinel.event
-        m_handler = mock.Mock()
-        m_handler.side_effect = _EX1()
-        m_count.return_value = list(range(1, 5))
-        retry = h_retry.Retry(m_handler, exceptions=(_EX11, _EX2))
-
-        retry(event)
-
-        m_handler.assert_called_with(event)
-        m_handler.set_health_status.assert_called_with(healthy=False)
-        m_sleep.assert_not_called()
